/*******************************************************************************
 * Copyright (c) 2013, Perforce Software
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/
package com.perforce.p4java.extension.operation;

import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.EnumSet;

import com.perforce.p4java.extension.command.PerforceDomain;
import com.perforce.p4java.extension.command.RequestImpl;
import com.perforce.p4java.extension.exception.ServerAccessException;
import com.perforce.p4java.extension.exception.ServerConnectionException;
import com.perforce.p4java.extension.exception.ServerErrorException;
import com.perforce.p4java.extension.path.PathStringFormat;
import com.perforce.p4java.extension.path.PerforcePath;
import com.perforce.p4java.extension.server.Server;
import com.perforce.p4java.extension.utility.Validate;

/**
 * 
 */
public class FileStreamCommand extends RequestImpl {

    private final PerforcePath depotPath;
    private Charset charset;

    public FileStreamCommand(PerforcePath depotPath) {
	this.depotPath = depotPath;
    }

    public FileStreamCommand(PerforcePath depotPath, Charset charset) {
	this(depotPath);
	this.charset = charset;
    }

    public static InputStream getFileStream(PerforcePath path, Charset charset,
	    Server server) throws ServerConnectionException,
	    ServerAccessException, ServerErrorException {
	Validate.notNull(path,
		"depotPath must not be null to FileStreamCommand.getFileStream");
	Validate.notNull(server,
		"server must not be null to FileStreamCommand.getFileStream");
	FileStreamCommand cmd = new FileStreamCommand(path, charset);
	return server.executeStreamCommand(cmd);
    }

    public static InputStream getFileStream(PerforcePath path, Server server)
	    throws ServerConnectionException, ServerAccessException,
	    ServerErrorException {
	Validate.notNull(path,
		"depotPath must not be null to FileStreamCommand.getFileStream");
	Validate.notNull(server,
		"server must not be null to FileStreamCommand.getFileStream");
	FileStreamCommand cmd = new FileStreamCommand(path);
	return server.executeStreamCommand(cmd);
    }

    /**
     * @see com.perforce.p4java.extension.command.Request#getCommand()
     */
    public String getCommand() {
	return PerforceDomain.FILE_CONTENTS.getCommand();
    }

    /**
     * @see com.perforce.p4java.extension.command.Request#getCommandArgs()
     */
    public String[] getCommandArgs() {
	return new String[] { "-q",
		depotPath.asString(EnumSet.of(PathStringFormat.ESCAPE)) };
    }

    /**
     * @see com.perforce.p4java.extension.command.Request#getInputString()
     */
    public String getInputString() {
	return depotPath.asString(EnumSet.of(PathStringFormat.ESCAPE));
    }

    @Override
    public Charset getCharset() {
	return this.charset;
    }
}
